package cmd

import (
	"acme-mana/src/common"
	"acme-mana/src/conf"
	"github.com/spf13/cobra"
	"os"
)

func InitCmd() (*cobra.Command, error) {
	rootCmd := &cobra.Command{
		Use:   "acme-mana",
		Short: "ACME协议客户端",
		Long:  "基于 ACME 协议实现自动申请证书和续签证书",
	}
	common.RootCmd = rootCmd

	rootCmd.AddCommand(initConfCmd())
	rootCmd.AddCommand(initServerCmd())
	rootCmd.AddCommand(initTaskCmd())
	rootCmd.AddCommand(initProviderCmd())
	rootCmd.AddCommand(certCmd())
	rootCmd.AddCommand(acmeCmd())

	flags := rootCmd.PersistentFlags()
	flags.BoolP("force", "f", false, "强制执行")
	flags.StringP("conf", "c", "config.yml", "指定配置文件")
	_ = rootCmd.ParseFlags(os.Args)

	//rootCmd.GenBashCompletion(os.Stdout)
	//rootCmd.GenPowerShellCompletion(os.Stdout)
	//rootCmd.GenZshCompletion(os.Stdout)

	conf.InitAppConfig()
	err := rootCmd.Execute()

	return rootCmd, err
}

func initConfCmd() *cobra.Command {
	confCmd := &cobra.Command{
		Use:   "conf",
		Short: "应用配置命令",
		Long:  "对当前应用配置进行操作",
		Run: func(cmd *cobra.Command, args []string) {
			cmd.Help()
		},
	}
	confShow := &cobra.Command{
		Use:   "show",
		Short: "查看配置列表",
		Long:  "查看系统配置列表",
		Run: func(cmd *cobra.Command, args []string) {
			confShow(cmd, args)
		},
	}
	confCmd.AddCommand(confShow)
	return confCmd
}

func initServerCmd() *cobra.Command {
	serverCmd := &cobra.Command{
		Use:   "server",
		Short: "服务相关命令",
		Long:  "配置服务相关参数, 如监听端口,监听地址等",
		Run: func(cmd *cobra.Command, args []string) {
			cmd.Help()
		},
	}
	editCmd := &cobra.Command{
		Use:   "edit",
		Short: "编辑服务配置",
		Long:  "编辑服务配置",
		Run: func(cmd *cobra.Command, args []string) {
			editServer(cmd, args)
		},
	}
	stateCmd := &cobra.Command{
		Use:   "state",
		Short: "查看服务状态",
		Long:  "查看服务状态",
		Run: func(cmd *cobra.Command, args []string) {
			serverState(cmd, args)
		},
	}
	startCmd := &cobra.Command{
		Use:   "start",
		Short: "启动服务",
		Long:  "启动服务",
		Run: func(cmd *cobra.Command, args []string) {
			startServer(cmd, args)
		},
	}
	stopCmd := &cobra.Command{
		Use:   "stop",
		Short: "停止服务",
		Long:  "停止服务",
		Run: func(cmd *cobra.Command, args []string) {
			stopServer(cmd, args)
		},
	}

	serverCmd.AddCommand(editCmd)
	serverCmd.AddCommand(stateCmd)
	serverCmd.AddCommand(startCmd)
	serverCmd.AddCommand(stopCmd)
	return serverCmd
}

func initTaskCmd() *cobra.Command {
	taskCmd := &cobra.Command{
		Use:   "task",
		Short: "定时任务相关命令",
		Long:  "自动刷新证书定时任务的相关命令",
		Run: func(cmd *cobra.Command, args []string) {
			cmd.Help()
		},
	}
	editTask := &cobra.Command{
		Use:   "edit",
		Short: "编辑定时任务配置",
		Long:  "编辑定时任务配置",
		Run: func(cmd *cobra.Command, args []string) {
			editTask(cmd, args)
		},
	}
	startTask := &cobra.Command{
		Use:   "start",
		Short: "启动定时任务",
		Long:  "启动定时任务",
		Run: func(cmd *cobra.Command, args []string) {
			startTask(cmd, args)
		},
	}
	stopTask := &cobra.Command{
		Use:   "stop",
		Short: "停止定时任务",
		Long:  "停止定时任务",
		Run: func(cmd *cobra.Command, args []string) {
			stopTask(cmd, args)
		},
	}
	statusTask := &cobra.Command{
		Use:   "status",
		Short: "查看定时任务状态",
		Long:  "查看定时任务状态",
		Run: func(cmd *cobra.Command, args []string) {
			statusTask(cmd, args)
		},
	}
	runTask := &cobra.Command{
		Use:   "run",
		Short: "手动执行一次任务",
		Long:  "手动执行一次任务",
		Run: func(cmd *cobra.Command, args []string) {
			runTask(cmd, args)
		},
	}
	taskCmd.AddCommand(editTask)
	taskCmd.AddCommand(startTask)
	taskCmd.AddCommand(stopTask)
	taskCmd.AddCommand(statusTask)
	taskCmd.AddCommand(runTask)
	return taskCmd
}

func initProviderCmd() *cobra.Command {
	providerCmd := &cobra.Command{
		Use:   "provider",
		Short: "DNS服务商相关命令",
		Long:  "DNS服务商相关命令",
		Run: func(cmd *cobra.Command, args []string) {
			cmd.Help()
		},
	}
	providerSupport := &cobra.Command{
		Use:   "support",
		Short: "列出支持的DNS服务商",
		Long:  "列出支持的DNS服务商",
		Run: func(cmd *cobra.Command, args []string) {
			providerSupport(cmd, args)
		},
	}
	providerList := &cobra.Command{
		Use:   "list",
		Short: "列出已配置的DNS服务商",
		Long:  "列出已配置的DNS服务商",
		Run: func(cmd *cobra.Command, args []string) {
			providerList(cmd, args)
		},
	}
	providerAdd := &cobra.Command{
		Use:   "add",
		Short: "添加DNS服务商",
		Long:  "添加DNS服务商",
		Run: func(cmd *cobra.Command, args []string) {
			providerAdd(cmd, args)
		},
	}
	providerEdit := &cobra.Command{
		Use:   "edit",
		Short: "编辑DNS服务商",
		Long:  "编辑DNS服务商",
		Run: func(cmd *cobra.Command, args []string) {
			providerEdit(cmd, args)
		},
	}
	providerDelete := &cobra.Command{
		Use:   "delete",
		Short: "删除DNS服务商",
		Long:  "删除DNS服务商",
		Run: func(cmd *cobra.Command, args []string) {
			providerDelete(cmd, args)
		},
	}

	providerCmd.AddCommand(providerSupport)
	providerCmd.AddCommand(providerList)
	providerCmd.AddCommand(providerAdd)
	providerCmd.AddCommand(providerEdit)
	providerCmd.AddCommand(providerDelete)
	return providerCmd
}

func certCmd() *cobra.Command {
	certCmd := &cobra.Command{
		Use:   "cert",
		Short: "证书配置相关命令",
		Long:  "证书配置相关命令",
		Run: func(cmd *cobra.Command, args []string) {
			cmd.Help()
		},
	}
	certList := &cobra.Command{
		Use:   "list",
		Short: "列出已配置的证书",
		Long:  "列出已配置的证书",
		Run: func(cmd *cobra.Command, args []string) {
			certList(cmd, args)
		},
	}
	certAdd := &cobra.Command{
		Use:   "add",
		Short: "添加证书",
		Long:  "添加证书",
		Run: func(cmd *cobra.Command, args []string) {
			certAdd(cmd, args)
		},
	}
	certDelete := &cobra.Command{
		Use:   "delete",
		Short: "删除证书",
		Long:  "删除证书",
		Run: func(cmd *cobra.Command, args []string) {
			certDelete(cmd, args)
		},
	}
	editCert := &cobra.Command{
		Use:   "edit",
		Short: "编辑证书",
		Long:  "编辑证书",
		Run: func(cmd *cobra.Command, args []string) {
			editCert(cmd, args)
		},
	}
	certCmd.AddCommand(certList)
	certCmd.AddCommand(certAdd)
	certCmd.AddCommand(certDelete)
	certCmd.AddCommand(editCert)
	return certCmd
}

func acmeCmd() *cobra.Command {
	acmeCmd := &cobra.Command{
		Use:   "acme",
		Short: "ACME相关命令",
		Long:  "ACME相关命令",
		Run: func(cmd *cobra.Command, args []string) {
			cmd.Help()
		},
	}
	return acmeCmd
}
