package conf

import (
	"acme-mana/src/common"
	"acme-mana/src/model"
	"acme-mana/src/util"
	"gopkg.in/yaml.v3"
	"log"
	"os"
)

func readAppConfig(c *model.AppConfig) *model.AppConfig {
	confFile, err := common.RootCmd.PersistentFlags().GetString("conf")
	if err != nil {
		log.Fatalln("读取配置文件参数失败")
	}

	_, err = os.Stat(confFile)
	if os.IsNotExist(err) {
		log.Println("配置文件不存在, 自动创建")
		writeConf(c, confFile)
	}

	file, err := os.ReadFile(confFile)
	if err != nil {
		log.Fatalln("读取配置文件失败")
	}
	//var conf = &model.AppConfig{}
	err = yaml.Unmarshal(file, c)
	if err != nil {
		log.Fatalln("解析配置文件失败")
	}
	return c
}

func InitAppConfig() {
	common.AppConf = DefaultAppConfig()
}

func RefreshConfig() {
	readAppConfig(common.AppConf)
}

func WriteConfig() {
	confFile, err := common.RootCmd.PersistentFlags().GetString("conf")
	if err != nil {
		log.Fatalln("读取配置文件参数失败")
	}
	writeConf(common.AppConf, confFile)
}

func writeConf(c *model.AppConfig, file string) {
	out, err := yaml.Marshal(c)
	if err != nil {
		log.Fatalln("序列化配置文件失败")
	}
	util.MkFileDir(file)
	err = os.WriteFile(file, out, 0644)
	if err != nil {
		log.Fatalln("写入配置文件失败")
	}
}

func DefaultAppConfig() *model.AppConfig {
	return &model.AppConfig{
		Server: &model.ServerConf{
			Host: "0.0.0.0",
			Port: 36851,
		},
		Task: &model.TaskConf{
			Delay:    0,
			Interval: 0,
		},
		Certs:     nil,
		Providers: nil,
	}
}
